@@ -249,7 +249,7 @@ min.advector <- function(..., na.rm = FALSE) {
     if (length(x) == 0) return (advector(Inf))
     Reduce1(x, "min")
 }
-##' @describeIn ADvector Equivalent of \link[base]{min}.
+##' @describeIn ADvector Equivalent of \link[base]{max}.
 max.advector <- function(..., na.rm = FALSE) {
     x <- c(...)
     if (na.rm) {
@@ -332,6 +332,11 @@ print.advector <- function (x, ...)  {
         x <- activate(x)
     }
     y <- f(x)
+    ## Return value must be real
+    if (inherits(y, "adcomplex")) {
+        stop("Return value 'adcomplex' not allowed. ",
+             "Please use extractors Re(.), Im(.), Mod(.)")
+    }
     ## Result might be sparse matrix => Store pattern as an attribute
     Pattern <- NULL
     if (inherits(y, "adsparse")) {
@@ -419,11 +424,34 @@ MakeTape <- function(f, x) {
             data.frame = function() {
                 get_df(.pointer(mod))
             },
+            force.update = function() {
+                force_update(.pointer(mod))
+            },
+            reorder = function(last = seq_len(mod$domain())) {
+                reorder_transform(.pointer(mod), last)
+            },
             node = function(i) {
                 mod <- .copy(mod)
                 get_node(.pointer(mod), i)
                 .expose(mod)
             },
+            timer = function(rep=100L, table=TRUE, inner=FALSE) {
+                ans <- .timer(.pointer(mod), rep) / rep
+                if (table) {
+                    tab1 <- as.matrix(xtabs(ans ~ names(ans)))
+                    colnames(tab1) <- "elapsed"
+                    if (inner) {
+                        i <- which(diff(names(ans)=="InvOp")==1)
+                        if (length(i) != 1)
+                            stop("Failed to locate inner updates")
+                        ans[1:i] <- 0
+                        tab2 <- as.matrix(xtabs(ans ~ names(ans)))
+                        colnames(tab2) <- "inner"
+                    } else tab2 <- NULL
+                    ans <- cbind(tab1, tab2)
+                }
+                ans
+            },
             par = mod$domainvec
         ),
         class="Tape")
@@ -495,6 +523,7 @@ print.Tape <- function(x,...){
     mod$newton(random, cfg)
     .expose(mod)
 }
+.timer <- timer
 ##' @describeIn Tape Global configuration parameters of the tape (experts only!)
 ##' \bold{comparison} By default, AD comparison gives an error
 ##' (\code{comparison="forbid"}).
@@ -507,17 +536,39 @@ print.Tape <- function(x,...){
 ##' @param comparison Set behaviour of AD comparison (\code{">"},\code{"=="}, etc).
 ##' @param atomic Set behaviour of AD BLAS operations (notably matrix multiply).
 ##' @param vectorize Enable/disable AD vectorized 'Ops' and 'Math'.
-TapeConfig <- function(comparison = c("NA", "forbid", "tape", "allow"),
+TapeConfig <- function(...,
+                       comparison = c("NA", "forbid", "tape", "allow"),
                        atomic = c("NA", "enable", "disable"),
                        vectorize = c("NA", "disable", "enable")) {
-    if (missing(comparison) || !is.integer(comparison))
-        comparison <- c("NA"=-1L, forbid=0L, tape=1L, allow=2L)[match.arg(comparison)]
-    if (missing(atomic) || !is.integer(atomic))
-        atomic <- c("NA"=-1L, enable=1L, disable=0L)[match.arg(atomic)]
-    if (missing(vectorize) || !is.integer(vectorize))
-        vectorize <- c("NA"=-1L, enable=1L, disable=0L)[match.arg(vectorize)]
-    ans <- unlist(set_tape_config(comparison, atomic, vectorize))
-    invisible(ans)
+    dotargs <- list(...)
+    if (length(dotargs) && is.list(dotargs[[1]])) {
+        if (length(dotargs) != 1)
+            stop("List argument must be the *only* argument")
+        if (!(missing(comparison) && missing(atomic) && missing(vectorize)))
+            stop("List argument must be the *only* argument")
+        return (do.call(set_tape_config, dotargs[[1]]))
+    }
+    args <- set_tape_config() ## Current settings
+    ## Set comparison
+    comparison <- match.arg(comparison)
+    comparison <- c("NA"="NA", "forbid"="forbid", "tape"="taped", "allow"="allow")[comparison]
+    args$compare <- comparison
+    atomic <- match.arg(atomic)
+    ## Set atomic
+    atomic <- match.arg(atomic)
+    atomic <- c("NA"="NA", "disable"="plain", "enable"="atomic")[atomic]
+    args$matmul <- atomic
+    args$mvnorm <- atomic
+    ## Set vectorize
+    vectorize <- match.arg(vectorize)
+    vectorize <- c("NA"="NA", "disable"="plain", "enable"="vectorize")[vectorize]
+    args$ops <- vectorize
+    args$math <- vectorize
+    args$sum <- vectorize
+    ## Set other flags
+    args[names(dotargs)] <- dotargs
+    ## return
+    invisible(do.call(set_tape_config, args))
 }
 
 ##' @describeIn Tape Move a chunk of data from R to the tape by evaluating a normal R function (replaces TMB functionality 'DATA_UPDATE').
@@ -526,11 +577,31 @@ TapeConfig <- function(comparison = c("NA", "forbid", "tape", "allow"),
 ##' F <- MakeTape(function(i) DataEval( function(i) rivers[i] , i), 1 )
 ##' F(1)
 ##' F(2)
+##' ## DATA_UPDATE example
+##' ## - Pay attention to lazy evaluation!
+##' ## - Use 'force.update()'
+##' mydat <- 1:4
+##' fetch <- function() { print("Getting 'mydat'"); .GlobalEnv$mydat }
+##' F <- MakeTape( function(x) x * sum(DataEval(fetch)), 1 )
+##' F$reorder() ## Re-order tape for faster execution
+##' F(1) ## No data update
+##' F(2) ## No data update
+##' mydat <- 5:8
+##' F(1) ## Still no data update !
+##' F$force.update() ## Signal that data has changed
+##' F(1) ## data update
+##' F(2) ## No data update
 DataEval <- function(f, x) {
-    if (ad_context())
+    if (ad_context()) {
+        if (missing(x))
+            x <- advector(numeric(0))
         TapedEval(f, x)
-    else
-        f(x)
+    } else {
+        if (missing(x))
+            f()
+        else
+            f(x)
+    }
 }
 
 ##' @describeIn Tape Extract tapes from a model object created by `MakeADFun`.
@@ -582,6 +653,13 @@ GetTape <- function(obj, name = c("ADFun", "ADGrad", "ADHess"), warn=TRUE) {
     }
     ans <- new(adfun)
     ans$copy(ADFun$ptr)
+    if (name == "ADHess") {
+        ## Set pattern (lower triangle only)
+        attr(ans, "Pattern") <- new("ngCMatrix",
+                                    i=env$Hrandom@i,
+                                    p=env$Hrandom@p,
+                                    Dim=env$Hrandom@Dim)
+    }
     .expose(ans)
 }
 
@@ -762,6 +840,19 @@ MakeADFun <- function(func, parameters, random=NULL, profile=NULL, integrate=NUL
     obj$env$data.term.indicator <- data.term.indicator
     if (length(observation.name) && !is.null(data[[observation.name]]))
         obj$env$data[[observation.name]] <- data[[observation.name]]
+    ## Send 'force update signal' to all known tapes
+    obj$force.update <- function() {
+        fu <- function(x) if (!is.null(x)) force_update(x)
+        fu(obj$env$ADFun$ptr)
+        fu(obj$env$ADGrad$ptr)
+        fu(environment(obj$env$spHess)$ADHess$ptr)
+        ## ridge.correct=TRUE uses a 4th tape (altHess)
+        if (!is.null(obj$env$altHess)) {
+            obj$env$altHess(TRUE)
+            fu(environment(obj$env$spHess)$ADHess$ptr)
+            obj$env$altHess(FALSE)
+        }
+    }
     ## Simulate
     obj$simulate <- function(par=obj$env$last.par,...) {
         clear_all()
