@@ -7,6 +7,10 @@
 ##' @param scale Extra scale parameter - see section 'Scaling'.
 ##' @return Vector of densities.
 dmvnorm <- function(x, mu=0, Sigma, log=FALSE, scale=1) {
+    if (length(x) == 0 && length(Sigma) == 0 && (!is.matrix(x) || ncol(x) == 0)) {
+        nout <- if (is.matrix(x)) nrow(x) else 1
+        return (rep (if (log) 0 else 1, nout))
+    }
     if (!identical(mu, 0)) {
         p <- length(mu)
         if (!all(dim(Sigma) == c(p, p))) stop("incompatible arguments")
@@ -39,6 +43,8 @@ dmvnorm <- function(x, mu=0, Sigma, log=FALSE, scale=1) {
     d <- nrow(Sigma)
     x0 <- as.vector(x) - as.vector(mu)
     dim(x0) <- c(d, length(x0) / d)
+    if (!is.null(keep))
+        dim(keep) <- dim(x0)
     anstype <- .anstype(x0, Sigma)
     anstype( dmvnorm0(advector(x0), advector(Sigma), log, keep) )
 }
@@ -223,7 +229,8 @@ dscale <- function(f, x, ...,
                     stop("'x' must be a matrix")
                 nc <- ncol(x)
                 if (length(scale) != nc)
-                    stop("Vector 'scale' must be compatible with *rows* of 'x'")
+                    stop(sprintf("Length of 'scale' vector (%d) must be either 'ncol(x)' (%d) or 'length(x)' (%d) ",
+                                 length(scale), nc, length(x)))
                 scale <- matrix(scale, nrow(x), ncol(x), byrow=TRUE)
             } else {
                 ## length(scale) == length(x)
