@@ -99,6 +99,24 @@ F$simplify("optimize")
 ## ----fig.cap="Tape of function after optimize"--------------------------------
 showGraph(F)
 
+## -----------------------------------------------------------------------------
+D <- MakeTape(diff, numeric(5))
+D
+
+## -----------------------------------------------------------------------------
+J <- D$jacfun()
+J
+
+## -----------------------------------------------------------------------------
+J(1:5)
+
+## -----------------------------------------------------------------------------
+Js <- D$jacfun(sparse=TRUE)
+Js
+
+## -----------------------------------------------------------------------------
+Js(1:5)
+
 ## -----------------------------------------------------------------------------
 f <- function(X) X %*% X
 
@@ -168,7 +186,7 @@ nldens <- function(obs, muX, muN) {
     ## 2
     L <- F$laplace(1:length(obs), SPA=TRUE)
     ## 3
-    L(rep(muX,0))
+    L(numeric(0))
 }
 
 ## -----------------------------------------------------------------------------
@@ -176,3 +194,29 @@ obj <- MakeADFun(function(p) nldens(x, p$muX, p$muN), list(muX=1, muN=1), silent
 opt <- nlminb(obj$par, obj$fn, obj$gr)
 sdreport(obj)
 
+## -----------------------------------------------------------------------------
+f <- function(x) {
+    ## Get real/imag part
+    xreal <- x[1]
+    ximag <- x[2]
+    ## Construct AD complex number
+    z <- as.complex(xreal) + 1i * as.complex(ximag)
+    ## Return real numbers
+    Mod(exp(z))
+}
+
+## -----------------------------------------------------------------------------
+F <- MakeTape(f, numeric(2))
+F
+
+## -----------------------------------------------------------------------------
+## Using R complex arithmetic
+f(1:2)
+
+## -----------------------------------------------------------------------------
+## Using the tape representation
+F(1:2)
+
+## -----------------------------------------------------------------------------
+F$print()
+
